# coding=utf-8
import mod.server.extraServerApi as serverApi
from math import floor
from mod.common.utils.mcmath import Vector3

CustomGoalCls = serverApi.GetCustomGoalCls()
CompFactory = serverApi.GetEngineCompFactory()


class PutBlock(CustomGoalCls):
    def __init__(self, entityId, argsJson):
        super(PutBlock, self).__init__(entityId, argsJson)
        self.mEntityId = entityId
        self.mTimeCounter = 0
        #
        self.mDimensionId = CompFactory.CreateDimension(self.mEntityId).GetEntityDimensionId()

    # region 继承函数
    def CanUse(self):
        if self._HasTarget() and self._IsTargetAlive() and (self._CheckHeightDifferenceWithTarget() or self._CheckPathAheadForFooting()):
            return True
        return False

    def CanContinueToUse(self):
        return self.CanUse()

    def CanBeInterrupted(self):
        return True

    def Start(self):
        self.mTimeCounter = 0
        CompFactory.CreateCommand(self.mEntityId).SetCommand("/say 开始搭方块")

    def Stop(self):
        CompFactory.CreateCommand(self.mEntityId).SetCommand("/say 结束搭方块")

    def Tick(self):
        self.mTimeCounter += 1
        perSec = self.mTimeCounter % 20 == 0
        if perSec:
            self._PutBlockToTarget()  # 每一秒尝试一次

    # endregion

    # region 类函数
    def _HasTarget(self):
        #是否有仇恨目标
        comp = CompFactory.CreateAction(self.mEntityId)
        targetId = comp.GetAttackTarget()
        hasTarget = targetId != "-1"
        return hasTarget

    def _IsTargetAlive(self):
        comp = CompFactory.CreateAction(self.mEntityId)
        targetId = comp.GetAttackTarget()
        comp = CompFactory.CreateGame(serverApi.GetLevelId())
        alive = comp.IsEntityAlive(targetId)
        return alive

    # 检查与目标之间是否存在高度差
    def _CheckHeightDifferenceWithTarget(self):
        comp = CompFactory.CreateAction(self.mEntityId)
        targetId = comp.GetAttackTarget()

        targetPos = CompFactory.CreatePos(targetId).GetFootPos()
        selfPos = CompFactory.CreatePos(self.mEntityId).GetFootPos()
        heightDifference = abs(targetPos[1] - selfPos[1])

        return heightDifference >= 1.0  # 因为是获取的 FootPos，所以这里的 2 就是 2 个方块的高度

    # 检查前往目标的路径，脚下是否有路
    def _CheckPathAheadForFooting(self):
        blockInfoComp = CompFactory.CreateBlockInfo(self.mEntityId)
        aheadBlockPos = self._GetPathAheadForFootingBlockPos()
        blockDict = blockInfoComp.GetBlockNew(aheadBlockPos, self.mDimensionId)

        return not blockDict or blockDict['name'] == 'minecraft:air'

    # 获取前方脚下的方块位置（这个是不依赖导航，前往目标的最近位置）
    def _GetPathAheadForFootingBlockPos(self):
        comp = CompFactory.CreateAction(self.mEntityId)
        targetId = comp.GetAttackTarget()

        targetPosX, targetPosY, targetPosZ = CompFactory.CreatePos(targetId).GetFootPos()
        entityPosX, entityPosY, entityPosZ = CompFactory.CreatePos(self.mEntityId).GetFootPos()
        # 去掉 y 方向上的不同，因为我们是要检测的是目标前方脚下一格的位置
        diff = Vector3(targetPosX - entityPosX, 0, targetPosZ - entityPosZ).Normalized()
        result = (entityPosX + diff[0], entityPosY - 0.5, entityPosZ + diff[2])
        result = tuple(map(int, map(floor, result)))  # 把实体坐标转换成方块坐标
        return result

    # 向目标搭建搭建方块
    def _PutBlockToTarget(self):
        if self._CheckHeightDifferenceWithTarget():
            # 第一种情况：存在高度差，那么就需要实体自己跳一下，然后在脚下搭建一个方块
            self._JumpAndPutBlockOnFoot()
        else:
            # 第二种情况：那就是在前方搭建一个方块
            self._PutBlockToAheadFoot()

    def _JumpAndPutBlockOnFoot(self):
        resBlockPos = CompFactory.CreatePos(self.mEntityId).GetFootPos()
        resBlockPos = tuple(map(int, map(floor, resBlockPos)))  # 把实体坐标转换成方块坐标
        # 先把自己弹起来
        actionComp = CompFactory.CreateAction(self.mEntityId)
        actionComp.SetMobKnockback(0, 0, 0.65, 0.65, 1)

        # 需要等待实体跳起来之后再放置方块
        CompFactory.CreateGame(self.mEntityId).AddTimer(0.3, self._PutBlock, resBlockPos)

    def _PutBlockToAheadFoot(self):
        resBlockPos = self._GetPathAheadForFootingBlockPos()
        self._PutBlock(resBlockPos)

    def _PutBlock(self, resBlockPos):
        CompFactory.CreateBlockInfo(self.mEntityId).SetBlockNew(resBlockPos, {'name': "minecraft:stone"}, 0, self.mDimensionId)

    # endregion
